﻿using NUnit.Framework;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UglyTrivia;

namespace Trivia.Tests
{
    [TestFixture]
    public class TriviaTests
    {
        [Test]
        public void Given_PlayerNotInBox_WhenPlayerAnswersCorrectly_ThenTheResponseContainsTypo()
        {
            var writer = new StringWriter();
            Console.SetOut(writer);
            Game aGame = new Game(new InMemoryGameStore());
            aGame.add("Chet");

            aGame.wasCorrectlyAnswered();

            Assert.That(writer.GetStringBuilder().ToString(), Is.Not.StringContaining("corrent"));
        }

        [Test]
        public void Given_NewInstanceOfGame_When_IPassStoreWithUsers_ThenTheGameContainsUsers()
        {
            var inMemoryGameStore = new InMemoryGameStore();
            var game = new Game(inMemoryGameStore);
            game.add("Peter");

            game = null;

            var newGame = new Game(inMemoryGameStore);
            var writer = new StringWriter();
            Console.SetOut(writer); 
            newGame.wasCorrectlyAnswered();

            Assert.That(writer.GetStringBuilder().ToString(),
                Is.StringContaining("Peter"));
        }

        [Test]
        public void Given_IHaveNewGameWithStoreContainingUsers_WhenICallCorrectAnswer_ThenNoEceptionThrown()
        {
            var writer = new StringWriter();
            Console.SetOut(writer);
            var inMemoryGameStore = new InMemoryGameStore();
            inMemoryGameStore.SaveUserName(0, "Peter");
            var game = new Game(inMemoryGameStore);

            game.wasCorrectlyAnswered();

            Assert.That(writer.GetStringBuilder().ToString(),
                Is.StringContaining("Peter"));
        }

        [Test]
        public void
            Given_IhaveUserWhoAnsweredCorrectly_When_IRecreateTheGameAndAnswerCorrectlyAgain_ThenTheUserShouldAnsweredCorrectlyTwice()
        {
            var writer = new StringWriter();
            Console.SetOut(writer);
            var inMemoryGameStore = new InMemoryGameStore();
            var game = new Game(inMemoryGameStore);
            game.add("Peter");
            game.wasCorrectlyAnswered();
            game = null;
            var newGame = new Game(inMemoryGameStore);
            newGame.wasCorrectlyAnswered();

            Assert.That(writer.GetStringBuilder().ToString(),
                Is.StringContaining("Peter now has 2 Gold Coins."));
        }

        [Test]
        public void Given_IHaveNoUserAdded_When_IAddUser_Then_TheUserCountIsOne()
        {
            var game = new Game(new InMemoryGameStore());
            game.add("Jakub");

            Assert.That(game.howManyPlayers(), Is.EqualTo(1));
        }
    }
}
